@@ -3,7 +3,7 @@ package YAHC;
 use strict;
 use warnings;
 
-our $VERSION = '0.005';
+our $VERSION = '0.006';
 
 use EV;
 use Time::HiRes;
@@ -38,8 +38,6 @@ sub YAHC::State::COMPLETED               () { 1 << 30 } # terminal state
 use constant {
     HTTP_PORT                  => 80,
     TCP_READ_CHUNK             => 65536,
-    RUN_UNTIL_ALL_CONNECTED    => 'run until all connected',
-    RUN_UNTIL_ALL_SENT_REQUEST => 'run until all sent request',
     CALLBACKS                  => [ qw/init_callback wait_synack_callback connected_callback
                                        writing_callback reading_callback completed_callback callback/ ],
 };
@@ -131,9 +129,12 @@ sub request {
 
     my $pool_args = $self->{pool_args};
     $request->{_target} = _wrap_target_selection($request->{host}) if $request->{host};
-    do { $request->{$_} ||= $pool_args->{$_} if $pool_args->{$_} } foreach (qw/host port/);
+    do { $request->{$_} ||= $pool_args->{$_} if $pool_args->{$_} } foreach (qw/host port scheme/);
     die 'YAHC: host must be defined' unless $request->{host};
 
+    $request->{scheme} //= 'http';
+    die 'YAHC: only support scheme http' unless $request->{scheme} eq 'http';
+
     my %callbacks;
     foreach (@{ CALLBACKS() }) {
         next unless $request->{$_};
@@ -187,8 +188,8 @@ sub yahc_conn_url {
     my $request = $_[0]->{request};
     return unless $target;
 
-    my ($host, $ip, $port, $protocol) = @{ $target };
-    return "$protocol://"
+    my ($host, $ip, $port) = @{ $target };
+    return $request->{scheme} . "://"
            . ($host || $ip)
            . ($port ne "80" ? ":$port" : '')
            . ($request->{path} || "/")
@@ -200,7 +201,7 @@ sub yahc_conn_url {
 ################################################################################
 
 sub _run {
-    my ($self, $how, $stop_condition) = @_;
+    my ($self, $how, $until_state, $until_state_connection_ids) = @_;
     die 'YAHC: storage object is destroyed' unless $self->{storage};
 
     if ($self->{pid} != $$) {
@@ -209,23 +210,22 @@ sub _run {
         $self->{loop}->loop_fork;
     }
 
-    if (defined $stop_condition) {
-        if ($stop_condition eq RUN_UNTIL_ALL_CONNECTED) {
-            $self->{stop_condition} = {
-                condition => RUN_UNTIL_ALL_CONNECTED,
-                expected_state => YAHC::State::CONNECTED(),
-                connections => { map { $_->id => 1 } grep { $_->state < YAHC::State::CONNECTED() } values %{ $self->{connections } } },
-            };
-        } elsif ($stop_condition eq RUN_UNTIL_ALL_SENT_REQUEST) {
-            $self->{stop_condition} = {
-                condition => RUN_UNTIL_ALL_SENT_REQUEST,
-                expected_state => YAHC::State::READING(),
-                connections => { map { $_->id => 1 } grep { $_->state < YAHC::State::WRITING() } values %{ $self->{connections } } },
-            };
+    if (defined $until_state) {
+        my $until_state_str = _strstate($until_state);
+        die "YAHC: unknown until_state $until_state" if $until_state_str =~ m/unknown/;
+
+        my @connections;
+        if (defined $until_state_connection_ids) {
+            @connections = @{ $until_state_connection_ids };
+            @connections = grep { exists $self->{connections}{$_} ? $self->{connections}{$_} : () } @connections;
         } else {
-            die "unknown stop_condition";
+            @connections = values %{ $self->{connections} }; # all connections
         }
 
+        $self->{stop_condition} = {
+            expected_state => $until_state,
+            connections => { map { $_->id => 1 } grep { $_->state < $until_state } @connections },
+        };
     } else {
         delete $self->{stop_condition};
     }
@@ -235,7 +235,7 @@ sub _run {
     return $loop->run($how || 0) unless $self->{debug}; # shortcut
 
     my $iterations = $loop->iteration;
-    _log_message('pid %d entering event loop%s', $$, $stop_condition ? " and $stop_condition" : '');
+    _log_message('pid %d entering event loop%s', $$, ($until_state ? " with until state " . _strstate($until_state) : ''));
     $loop->run($how || 0);
     _log_message('pid %d exited from event loop after %d iterations', $$, $loop->iteration - $iterations);
 }
@@ -253,12 +253,13 @@ sub _check_stop_condition {
 
     delete $stop_condition->{connections}{$conn->id};
     my $awaiting_connections = scalar keys %{ $stop_condition->{connections} };
+    my $expected_state = $stop_condition->{expected_state};
 
     if ($awaiting_connections == 0) {
-        $self->_break(sprintf("stop condition '%s' is met", $stop_condition->{condition}));
+        $self->_break(sprintf("until state '%s' is reached", _strstate($expected_state)));
     } else {
-        _log_message("Still have %d connections awaiting stop condition '%s'",
-                     $awaiting_connections, $stop_condition->{condition}) if $self->{debug};
+        _log_message("Still have %d connections awaiting state '%s'",
+                     $awaiting_connections, _strstate($expected_state)) if $self->{debug};
     }
 }
 
@@ -567,9 +568,8 @@ sub _get_next_target {
     $ip = $host if $host =~ m/^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$/o;
     $ip = inet_ntoa(gethostbyname($host) or die "Failed to resolve $host: $!") unless $ip;
     $port ||= $conn->{request}{port} || HTTP_PORT;
-    my $protocol = 'http';
 
-    return @{ $conn->{selected_target} = [ $host, $ip, $port, $protocol ] };
+    return @{ $conn->{selected_target} = [ $host, $ip, $port ] };
 }
 
 ################################################################################
